from __future__ import unicode_literals


import os
import unittest

from flask import Flask, Blueprint, render_template_string
from flask_cdn import CDN


class DefaultsTest(unittest.TestCase):
    def setUp(self):
        self.app = Flask(__name__)
        self.app.testing = True

        CDN(self.app)

    def test_debug_default(self):
        """ Tests CDN_DEBUG default value is correctly set. """
        self.assertEquals(self.app.config['CDN_DEBUG'], False)

    def test_domain_default(self):
        """ Tests CDN_DOMAIN default value is correctly set. """
        self.assertEquals(self.app.config['CDN_DOMAIN'], None)

    def test_https_default(self):
        """ Tests CDN_HTTPS default value is correctly set. """
        self.assertEquals(self.app.config['CDN_HTTPS'], None)

    def test_timestamp_default(self):
        """ Tests CDN_TIMESTAMP default value is correctly set. """
        self.assertEquals(self.app.config['CDN_TIMESTAMP'], True)


class UrlTests(unittest.TestCase):
    def setUp(self):
        self.app = Flask(__name__)
        self.app.testing = True

        self.app.config['CDN_DOMAIN'] = 'mycdnname.cloudfront.net'
        self.app.config['CDN_TIMESTAMP'] = False

        @self.app.route('/<url_for_string>')
        def a(url_for_string):
            return render_template_string(url_for_string)

        @self.app.route('/')
        def b():
            return render_template_string("{{ url_for('b') }}")

    def client_get(self, ufs, secure=False):
        CDN(self.app)
        client = self.app.test_client()
        if secure:
            return client.get('/%s' % ufs, base_url='https://localhost')
        else:
            return client.get('/%s' % ufs)

    def test_url_for(self):
        """ Tests static endpoint correctly affects generated URLs. """
        # non static endpoint url_for in template
        self.assertEquals(self.client_get('').get_data(True), '/')

        # static endpoint url_for in template
        ufs = "{{ url_for('static', filename='bah.js') }}"
        exp = 'http://mycdnname.cloudfront.net/static/bah.js'
        self.assertEquals(self.client_get(ufs).get_data(True), exp)

    def test_url_for_debug(self):
        """ Tests app.debug correctly affects generated URLs. """
        self.app.config['CDN_DEBUG'] = True
        ufs = "{{ url_for('static', filename='bah.js') }}"

        exp = '/static/bah.js'
        self.assertEquals(self.client_get(ufs).get_data(True), exp)

    def test_url_for_https(self):
        """ Tests CDN_HTTPS correctly affects generated URLs. """
        ufs = "{{ url_for('static', filename='bah.js') }}"

        https_exp = 'https://mycdnname.cloudfront.net/static/bah.js'
        http_exp = 'http://mycdnname.cloudfront.net/static/bah.js'

        self.app.config['CDN_HTTPS'] = True
        self.assertEquals(self.client_get(ufs, secure=True).get_data(True),
                          https_exp)
        self.assertEquals(self.client_get(ufs).get_data(True), https_exp)

        self.app.config['CDN_HTTPS'] = False
        self.assertEquals(self.client_get(ufs, secure=True).get_data(True),
                          http_exp)
        self.assertEquals(self.client_get(ufs).get_data(True), http_exp)

        self.app.config['CDN_HTTPS'] = None
        self.assertEquals(self.client_get(ufs, secure=True).get_data(True),
                          https_exp)
        self.assertEquals(self.client_get(ufs).get_data(True), http_exp)

    def test_url_for_timestamp(self):
        """ Tests CDN_TIMESTAMP correctly affects generated URLs. """
        ufs = "{{ url_for('static', filename='bah.js') }}"

        self.app.config['CDN_TIMESTAMP'] = True
        path = os.path.join(self.app.static_folder, 'bah.js')
        ts = int(os.path.getmtime(path))
        exp = 'http://mycdnname.cloudfront.net/static/bah.js?t={0}'.format(ts)
        self.assertEquals(self.client_get(ufs).get_data(True), exp)

        self.app.config['CDN_TIMESTAMP'] = False
        exp = 'http://mycdnname.cloudfront.net/static/bah.js'
        self.assertEquals(self.client_get(ufs).get_data(True), exp)

    def test_for_scheme(self):
        """ Tests _scheme correctly overrides CDN_HTTPS option. """
        ufs = "{{ url_for('static', filename='bah.js', _scheme='https') }}"
        self.app.config['CDN_HTTPS'] = False
        exp = 'https://mycdnname.cloudfront.net/static/bah.js'
        self.assertEquals(self.client_get(ufs, secure=False).get_data(True),
                          exp)

        ufs = "{{ url_for('static', filename='bah.js', _scheme='http') }}"
        self.app.config['CDN_HTTPS'] = True
        exp = 'http://mycdnname.cloudfront.net/static/bah.js'
        self.assertEquals(self.client_get(ufs, secure=True).get_data(True),
                          exp)

        ufs = "{{ url_for('static', filename='bah.js', _scheme=None) }}"
        self.app.config['CDN_HTTPS'] = True
        exp = '//mycdnname.cloudfront.net/static/bah.js'
        self.assertEquals(self.client_get(ufs, secure=True).get_data(True),
                          exp)


class BlueprintTest(unittest.TestCase):
    def setUp(self):
        self.app = Flask(__name__)
        self.app.testing = True
        self.app.config['CDN_DOMAIN'] = 'mycdnname.cloudfront.net'
        self.app.config['CDN_TIMESTAMP'] = True
        CDN(self.app)

        test_bp = Blueprint('test_bp', 'test')
        @test_bp.route('/without_static/<url_for_string>')
        def a(url_for_string):
            return render_template_string(url_for_string)
        self.app.register_blueprint(test_bp)

        test2_bp = Blueprint('test2_bp', 'test2', static_folder=self.app.static_folder + '_bp')
        @test2_bp.route('/with_static/<url_for_string>')
        def b(url_for_string):
            return render_template_string(url_for_string)
        self.app.register_blueprint(test2_bp)


    def test_blueprint_without_static_folder(self):
        ufs = "{{ url_for('static', filename='bah.js') }}"
        client = self.app.test_client()
        response = client.get('/without_static/%s' % ufs)
        path = os.path.join(self.app.static_folder, 'bah.js')
        ts = int(os.path.getmtime(path))
        exp = 'http://mycdnname.cloudfront.net/static/bah.js?t={0}'.format(ts)
        self.assertEquals(response.get_data(True), exp)


    def test_blueprint_with_static_folder(self):
        ufs = "{{ url_for('static', filename='bah_bp.js') }}"
        client = self.app.test_client()
        response = client.get('/with_static/%s' % ufs)
        path = os.path.join(self.app.blueprints['test2_bp'].static_folder, 'bah_bp.js')
        ts = int(os.path.getmtime(path))
        exp = 'http://mycdnname.cloudfront.net/static/bah_bp.js?t={0}'.format(ts)
        self.assertEquals(response.get_data(True), exp)


if __name__ == '__main__':
    unittest.main()



import os

from flask import url_for as flask_url_for
from flask import current_app, request


def url_for(endpoint, **values):
    """
    Generates a URL to the given endpoint.

    If the endpoint is for a static resource then a URL to the CDN is
    generated, otherwise the call is passed on to `flask.url_for`.

    Because this function is set as a jinja environment variable when
    `CDN.init_app` is invoked, this function replaces `flask.url_for` in
    templates automatically. It is unlikely that this function will need to be
    directly called from within your application code, unless you need to refer
    to static assets outside of your templates.
    """
    app = current_app

    if app.config['CDN_DEBUG']:
        return flask_url_for(endpoint, **values)

    def endpoint_match(endpoint):
        if endpoint in app.config['CDN_ENDPOINTS']:
            return True

        for x in app.config['CDN_ENDPOINTS']:
            if endpoint.endswith('.%s' % x):
                return True

        return False

    if endpoint_match(endpoint):
        try:
            scheme = values.pop('_scheme')
        except KeyError:
            scheme = 'http'
            cdn_https = app.config['CDN_HTTPS']
            if cdn_https is True or (cdn_https is None and request.is_secure):
                scheme = 'https'

        static_folder = app.static_folder
        if (request.blueprint is not None and
                request.blueprint in app.blueprints and
                app.blueprints[request.blueprint].has_static_folder):
            static_folder = app.blueprints[request.blueprint].static_folder

        urls = app.url_map.bind(app.config['CDN_DOMAIN'], url_scheme=scheme)

        if app.config['CDN_TIMESTAMP']:
            path = os.path.join(static_folder, values['filename'])
            values['t'] = int(os.path.getmtime(path))

        return urls.build(endpoint, values=values, force_external=True)

    return flask_url_for(endpoint, **values)


class CDN(object):
    """
    The CDN object allows your application to use Flask-CDN.

    When initialising a CDN object you may optionally provide your
    :class:`flask.Flask` application object if it is ready. Otherwise,
    you may provide it later by using the :meth:`init_app` method.

    :param app: optional :class:`flask.Flask` application object
    :type app: :class:`flask.Flask` or None
    """
    def __init__(self, app=None):
        """
        An alternative way to pass your :class:`flask.Flask` application
        object to Flask-CDN. :meth:`init_app` also takes care of some
        default `settings`_.

        :param app: the :class:`flask.Flask` application object.
        """
        self.app = app
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        defaults = [('CDN_DEBUG', app.debug),
                    ('CDN_DOMAIN', None),
                    ('CDN_HTTPS', None),
                    ('CDN_TIMESTAMP', True),
                    ('CDN_ENDPOINTS', ['static'])]

        for k, v in defaults:
            app.config.setdefault(k, v)

        if app.config['CDN_DOMAIN']:
            app.jinja_env.globals['url_for'] = url_for



import setuptools


setuptools.setup(
    name='Flask-CDN',
    version='1.5.2',
    url='https://libwilliam.github.io/flask-compress/',
    license='MIT',
    author='William Fagan',
    author_email='libwilliam@gmail.com',
    description='Serve the static files in your Flask app from a CDN.',
    long_description='Full documentation can be found on the Flask-CDN "Home Page".',
    py_modules=['flask_cdn'],
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=[
        'Flask'
    ],
    test_suite='tests',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)
